@@ -7,21 +7,11 @@ static str tmp;
 
 static unsigned long databytes = 0;
 
-const response* backend_validate_init(void)
-{
-  return 0;
-}
-
 void backend_handle_reset(void)
 {
   databytes = 0;
 }
 
-const response* backend_validate_sender(str* sender)
-{
-  return 0;
-}
-
 const response* backend_handle_sender(str* sender)
 {
   str_copys(&tmp, "Sender='");
@@ -31,11 +21,6 @@ const response* backend_handle_sender(str* sender)
   return &r;
 }
 
-const response* backend_validate_recipient(str* recipient)
-{
-  return 0;
-}
-
 const response* backend_handle_recipient(str* recipient)
 {
   str_copys(&tmp, "Recipient='");
