@@ -8,11 +8,12 @@
 #include "mailrules.h"
 #include "smtp.h"
 
-static RESPONSE(badbounce, 550, "Bounce messages should have a single recipient.");
-static RESPONSE(too_long, 552, "Sorry, that message exceeds the maximum message length.");
-static RESPONSE(hops, 554, "This message is looping, too many hops.");
-static RESPONSE(internal, 451, "Internal error.");
-static RESPONSE(manyrcpt, 550, "Too many recipients");
+static RESPONSE(too_long, 552, "5.2.3 Sorry, that message exceeds the maximum message length.");
+static RESPONSE(hops, 554, "5.6.0 This message is looping, too many hops.");
+static RESPONSE(manyrcpt, 550, "5.5.3 Too many recipients");
+static RESPONSE(mustauth, 530, "5.7.1 You must authenticate first.");
+static RESPONSE(nodomain,554,"5.1.2 Address is missing a domain name");
+static RESPONSE(nofqdn,554,"5.1.2 Address does not contain a fully qualified domain name");
 
 const char UNKNOWN[] = "unknown";
 
@@ -39,6 +40,18 @@ static const char* remote_ip;
 static str fixup_host;
 static str fixup_ip;
 static const char* linkproto;
+static const char* require_auth;
+
+static const response* check_fqdn(const str* s)
+{
+  int at;
+  int dot;
+  if ((at = str_findlast(s, '@')) <= 0)
+    return &resp_nodomain;
+  if ((dot = str_findlast(s, '.')) < at)
+    return &resp_nofqdn;
+  return 0;
+}
 
 const char* getprotoenv(const char* name)
 {
@@ -56,6 +69,7 @@ const response* handle_init(void)
 
   set_timeout();
 
+  require_auth = getenv("REQUIRE_AUTH");
   if ((linkproto = getenv("PROTO")) == 0) linkproto = "TCP";
   if ((local_ip = getprotoenv("LOCALIP")) != 0 && *local_ip == 0)
     local_ip = 0;
@@ -79,6 +93,7 @@ const response* handle_init(void)
   maxdatabytes = rules_getenvu("DATABYTES");
 
   if ((resp = backend_validate_init()) != 0) return resp;
+  if ((resp = cvm_validate_init()) != 0) return resp;
 
   return rules_init();
 }
@@ -93,20 +108,12 @@ const response* handle_reset(void)
   return 0;
 }
 
-int number_ok(const response* resp)
-{
-  return resp->number < 400;
-}
-
-int response_ok(const response* resp)
-{
-  return resp == 0 || number_ok(resp);
-}
-
 const response* handle_sender(str* sender)
 {
   const response* resp;
   const response* tmpresp;
+  if (require_auth && !(authenticated || relayclient != 0))
+    return &resp_mustauth;
   /* Logic:
    * if rules_validate_sender returns a response, use it
    * else if backend_validate_sender returns a response, use it
@@ -115,6 +122,8 @@ const response* handle_sender(str* sender)
   maxrcpts = rules_getenvu("MAXRCPTS");
   relayclient = rules_getenv("RELAYCLIENT");
   maxdatabytes = rules_getenvu("DATABYTES");
+  if (resp == 0 && sender->len > 0)
+    resp = check_fqdn(sender);
   if (resp == 0)
     resp = backend_validate_sender(sender);
   if (!response_ok(resp))
@@ -130,12 +139,14 @@ const response* handle_recipient(str* recip)
 {
   const response* resp;
   const response* hresp;
+  ++rcpt_count;
   if (maxrcpts > 0 && rcpt_count > maxrcpts) return &resp_manyrcpt;
-  if (++rcpt_count > 1 && is_bounce) return &resp_badbounce;
   if ((resp = rules_validate_recipient(recip)) != 0) {
     if (!number_ok(resp))
       return resp;
   }
+  else if ((resp = check_fqdn(recip)) != 0)
+    return resp;
   else if (relayclient != 0)
     str_cats(recip, relayclient);
   else if (authenticated)
@@ -144,6 +155,10 @@ const response* handle_recipient(str* recip)
     if (!number_ok(resp))
       return resp;
   }
+  else if ((resp = cvm_validate_recipient(recip)) != 0) {
+    if (!number_ok(resp))
+      return resp;
+  }
   if (!response_ok(hresp = backend_handle_recipient(recip)))
     return hresp;
   if (resp == 0) resp = hresp;
@@ -236,7 +251,6 @@ const response* handle_data_start(const char* helo_domain,
 {
   const response* resp;
   maxdatabytes = rules_getenvu("DATABYTES");
-  if (is_bounce && rcpt_count > 1) return &resp_badbounce;
   resp = backend_handle_data_start();
   if (response_ok(resp)) {
     received.len = 0;
