@@ -62,21 +62,21 @@ run_compare_test() {
 
 runtest() {
 cat >$tmp/rules <<EOF
-ka:b:K1:::HEADER_ADD="X-Header: testing!!!"
+ka@example.com:b@example.com:K1:::HEADER_ADD="X-Header: testing!!!"
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<a>
-RCPT TO:<b>
+MAIL FROM:<a@example.com>
+RCPT TO:<b@example.com>
 DATA
 .
 EOF
 
 sfecho <<EOF
-MAIL FROM:<a>
-RCPT TO:<c>
+MAIL FROM:<a@example.com>
+RCPT TO:<c@example.com>
 DATA
 .
 EOF
@@ -84,13 +84,13 @@ EOF
 rm -f $tmp/rules
 }
 run_compare_test tests/rules-header-add  <<END_OF_TEST_RESULTS
-250 Sender='a'.^M
+250 Sender='a@example.com'.^M
 250 K1^M
-354 End your message with a period.^M
+354 End your message with a period on a line by itself.^M
 250 Received 151 bytes.^M
-250 Sender='a'.^M
-250 Recipient='c'.^M
-354 End your message with a period.^M
+250 Sender='a@example.com'.^M
+250 Recipient='c@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 128 bytes.^M
 END_OF_TEST_RESULTS
 
@@ -107,7 +107,7 @@ MAILRULES=$tmp/rules
 
 sfecho <<EOF
 MAIL FROM:<>
-MAIL FROM:<foo>
+MAIL FROM:<foo@example.com>
 EOF
 
 rm -f $tmp/rules
@@ -122,33 +122,33 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-k*:one@one:KK
-d*:two@two:DD
-z*:three@three:ZZ
-zx@y:four@four:ZZZ
-p*:five@five:PP
+k*:one@one.example.com:KK
+d*:two@two.example.com:DD
+z*:three@three.example.com:ZZ
+zx@y:four@four.example.com:ZZZ
+p*:five@five.example.com:PP
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<nobody@nobody>
-RCPT TO:<one@one>
-RCPT TO:<two@two>
-RCPT TO:<three@three>
-RCPT TO:<four@four>
-RCPT TO:<five@five>
+MAIL FROM:<nobody@example.net>
+RCPT TO:<one@one.example.com>
+RCPT TO:<two@two.example.com>
+RCPT TO:<three@three.example.com>
+RCPT TO:<four@four.example.com>
+RCPT TO:<five@five.example.com>
 EOF
 
 rm -f $tmp/rules
 }
 run_compare_test tests/rules-recip  <<END_OF_TEST_RESULTS
-250 Sender='nobody@nobody'.^M
+250 Sender='nobody@example.net'.^M
 250 KK^M
 553 DD^M
 451 ZZ^M
-250 Recipient='four@four'.^M
-250 Recipient='five@five'.^M
+250 Recipient='four@four.example.com'.^M
+250 Recipient='five@five.example.com'.^M
 END_OF_TEST_RESULTS
 
 
@@ -203,15 +203,15 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-k!a@a:*:A
-k!!a@a:*:B
+k!a@example.com:*:A
+k!!a@example.com:*:B
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<a@a>
-MAIL FROM:<b@b>
+MAIL FROM:<a@example.com>
+MAIL FROM:<b@example.net>
 EOF
 
 rm -f $tmp/rules
@@ -232,26 +232,26 @@ d*:*:DD
 EOF
 
 cat <<EOF | cdbmake $tmp/list.cdb $tmp/list.tmp
-+3,0:a@a->
-+2,0:@c->
++13,0:a@example.net->
++12,0:@example.com->
 
 EOF
 
 cat <<EOF | cdbmake $tmp/atlist.cdb $tmp/atlist.tmp
-+1,0:b->
++11,0:example.org->
 
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<a@a>
-MAIL FROM:<b@a>
-MAIL FROM:<a@b>
-MAIL FROM:<b@B>
-MAIL FROM:<c@c>
-MAIL FROM:<c@C>
-MAIL FROM:<d@d>
+MAIL FROM:<a@example.net>
+MAIL FROM:<b@example.net>
+MAIL FROM:<a@example.org>
+MAIL FROM:<b@Example.ORG>
+MAIL FROM:<c@example.com>
+MAIL FROM:<c@Example.COM>
+MAIL FROM:<d@example.biz>
 EOF
 
 rm -f $tmp/rules $tmp/list.cdb $tmp/atlist.cdb
@@ -271,21 +271,21 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-kone@one:*:KK
-dtwo@two:*:DD
-zthree@three:*:ZZ
-zfour@four:one@one:ZZ
-pfive@five:*:PP
+kone@one.example.com:*:KK
+dtwo@two.example.com:*:DD
+zthree@three.example.com:*:ZZ
+zfour@four.example.com:one@one.example.com:ZZ
+pfive@five.example.com:*:PP
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<one@one>
-MAIL FROM:<two@two>
-MAIL FROM:<three@three>
-MAIL FROM:<four@four>
-MAIL FROM:<five@five>
+MAIL FROM:<one@one.example.com>
+MAIL FROM:<two@two.example.com>
+MAIL FROM:<three@three.example.com>
+MAIL FROM:<four@four.example.com>
+MAIL FROM:<five@five.example.com>
 EOF
 
 rm -f $tmp/rules
@@ -294,8 +294,20 @@ run_compare_test tests/rules-sender  <<END_OF_TEST_RESULTS
 250 KK^M
 553 DD^M
 451 ZZ^M
-250 Sender='four@four'.^M
-250 Sender='five@five'.^M
+250 Sender='four@four.example.com'.^M
+250 Sender='five@five.example.com'.^M
+END_OF_TEST_RESULTS
+
+
+##### Test tests/qmqpfront-echo #####
+
+runtest() {
+printf '42:5:test\n,13:a@example.com,13:b@example.com,,' \
+| $src/qmqpfront-echo 2>/dev/null
+echo
+}
+run_compare_test tests/qmqpfront-echo  <<END_OF_TEST_RESULTS
+20:KReceived 133 bytes.,
 END_OF_TEST_RESULTS
 
 
@@ -360,13 +372,13 @@ d
 EOF
 }
 run_compare_test tests/smtpfront-maxnotimpl  <<END_OF_TEST_RESULTS
-500 Not implemented.^M
-503-Too many unimplemented commands.^M
-503 Closing connection.^M
-500 Not implemented.^M
-500 Not implemented.^M
-500 Not implemented.^M
-500 Not implemented.^M
+500 5.5.1 Not implemented.^M
+503-5.5.0 Too many unimplemented commands.^M
+503 5.5.0 Closing connection.^M
+500 5.5.1 Not implemented.^M
+500 5.5.1 Not implemented.^M
+500 5.5.1 Not implemented.^M
+500 5.5.1 Not implemented.^M
 END_OF_TEST_RESULTS
 
 
@@ -381,7 +393,7 @@ PATTERNS=$tmp/patterns
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<nobody>
+RCPT TO:<nobody@example.com>
 DATA
 header1: data
 header2: another field
@@ -401,7 +413,7 @@ EOF
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<nobody>
+RCPT TO:<nobody@example.com>
 DATA
 header
 
@@ -415,13 +427,13 @@ rm -f $tmp/patterns
 }
 run_compare_test tests/patterns-header  <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 146 bytes.^M
 END_OF_TEST_RESULTS
 
@@ -430,30 +442,30 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-ka@a:a@b:K1
-ka@a:b@b:K2
-kb@a:a@b:K3
-kb@a:b@b:K4
+ka@example.net:a@example.com:K1
+ka@example.net:b@example.com:K2
+kb@example.net:a@example.com:K3
+kb@example.net:b@example.com:K4
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<a@a>
-RCPT TO:<a@b>
-RCPT TO:<b@b>
-MAIL FROM:<b@a>
-RCPT TO:<a@b>
-RCPT TO:<b@b>
+MAIL FROM:<a@example.net>
+RCPT TO:<a@example.com>
+RCPT TO:<b@example.com>
+MAIL FROM:<b@example.net>
+RCPT TO:<a@example.com>
+RCPT TO:<b@example.com>
 EOF
 
 rm -f $tmp/rules
 }
 run_compare_test tests/rules-both  <<END_OF_TEST_RESULTS
-250 Sender='a@a'.^M
+250 Sender='a@example.net'.^M
 250 K1^M
 250 K2^M
-250 Sender='b@a'.^M
+250 Sender='b@example.net'.^M
 250 K3^M
 250 K4^M
 END_OF_TEST_RESULTS
@@ -469,24 +481,24 @@ d*:*:DD
 EOF
 
 cat >$tmp/list <<EOF
-a@a
-@c
+a@example.net
+@example.com
 EOF
 
 cat >$tmp/atlist <<EOF
-b
+example.biz
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<a@a>
-MAIL FROM:<b@a>
-MAIL FROM:<a@b>
-MAIL FROM:<b@B>
-MAIL FROM:<c@c>
-MAIL FROM:<c@C>
-MAIL FROM:<d@d>
+MAIL FROM:<a@example.net>
+MAIL FROM:<b@example.net>
+MAIL FROM:<a@example.biz>
+MAIL FROM:<b@Example.BIZ>
+MAIL FROM:<c@example.com>
+MAIL FROM:<c@Example.COM>
+MAIL FROM:<d@example.org>
 EOF
 
 rm -f $tmp/rules $tmp/list $tmp/atlist
@@ -512,15 +524,15 @@ EOF
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<a@a>
-RCPT TO:<b@b>
+MAIL FROM:<a@example.com>
+RCPT TO:<b@example.net>
 EOF
 
 rm -f $tmp/rules
 }
 run_compare_test tests/rules-noop  <<END_OF_TEST_RESULTS
-250 Sender='a@a'.^M
-250 Recipient='b@b@rc'.^M
+250 Sender='a@example.com'.^M
+250 Recipient='b@example.net@rc'.^M
 END_OF_TEST_RESULTS
 
 
@@ -538,7 +550,7 @@ PATTERNS=$tmp/patterns
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<nobody>
+RCPT TO:<nobody@example.com>
 DATA
 before
 
@@ -550,8 +562,8 @@ rm -f $tmp/patterns
 }
 run_compare_test tests/patterns-message  <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 response 2^M
 END_OF_TEST_RESULTS
 
@@ -586,7 +598,7 @@ PATTERNS=$tmp/patterns
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<nobody>
+RCPT TO:<nobody@example.com>
 DATA
 before
 
@@ -598,8 +610,8 @@ rm -f $tmp/patterns
 }
 run_compare_test tests/patterns-normal  <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 END_OF_TEST_RESULTS
 
@@ -732,7 +744,7 @@ PATTERNS=$tmp/patterns
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<nobody>
+RCPT TO:<nobody@example.com>
 DATA
 Subject: $subject
 .
@@ -742,55 +754,80 @@ rm -f $tmp/patterns
 }
 run_compare_test tests/patterns-general 'xwordx' <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 END_OF_TEST_RESULTS
 
 run_compare_test tests/patterns-general 'word at the beginning' <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 END_OF_TEST_RESULTS
 
 run_compare_test tests/patterns-general 'last word' <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 END_OF_TEST_RESULTS
 
 run_compare_test tests/patterns-general 'middle word middle' <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 END_OF_TEST_RESULTS
 
 run_compare_test tests/patterns-general 'word' <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 END_OF_TEST_RESULTS
 
 run_compare_test tests/patterns-general 'xord' <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 142 bytes.^M
 END_OF_TEST_RESULTS
 
 
+##### Test tests/smtpfront-maxrcpts #####
+
+runtest() {
+MAXRCPTS=2
+export MAXRCPTS
+
+sfecho <<EOF
+MAIL FROM:<notbounce@example.com>
+RCPT TO:<addr1@example.net>
+RCPT TO:<addr2@example.net>
+RCPT TO:<addr3@example.net>
+RCPT TO:<addr4@example.net>
+EOF
+
+unset MAXRCPTS
+}
+run_compare_test tests/smtpfront-maxrcpts  <<END_OF_TEST_RESULTS
+250 Sender='notbounce@example.com'.^M
+250 Recipient='addr1@example.net'.^M
+250 Recipient='addr2@example.net'.^M
+550 5.5.3 Too many recipients^M
+550 5.5.3 Too many recipients^M
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/smtpfront-looping-received #####
 
 runtest() {
 MAXHOPS=1
 
 sfecho <<EOF
-MAIL FROM:<somebody>
-RCPT TO:<nobody@nowhere>
+MAIL FROM:<somebody@example.com>
+RCPT TO:<nobody@example.org>
 DATA
 Received: foo
 .
@@ -799,8 +836,8 @@ EOF
 echo
 
 sfecho <<EOF
-MAIL FROM:<somebody>
-RCPT TO:<nobody@nowhere>
+MAIL FROM:<somebody@example.com>
+RCPT TO:<nobody@example.org>
 DATA
 Received: foo
 Received: foo
@@ -808,15 +845,15 @@ Received: foo
 EOF
 }
 run_compare_test tests/smtpfront-looping-received  <<END_OF_TEST_RESULTS
-250 Sender='somebody'.^M
-250 Recipient='nobody@nowhere'.^M
-354 End your message with a period.^M
+250 Sender='somebody@example.com'.^M
+250 Recipient='nobody@example.org'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 142 bytes.^M
 
-250 Sender='somebody'.^M
-250 Recipient='nobody@nowhere'.^M
-354 End your message with a period.^M
-554 This message is looping, too many hops.^M
+250 Sender='somebody@example.com'.^M
+250 Recipient='nobody@example.org'.^M
+354 End your message with a period on a line by itself.^M
+554 5.6.0 This message is looping, too many hops.^M
 END_OF_TEST_RESULTS
 
 
@@ -827,6 +864,7 @@ local quit="$1"
 local command="$2"
 maildir
 tstmsg new/1
+tstmsg new/this.is.a.very.long.filename.that.should.get.truncated.after.the.X...XBUGBUGBUGBUG
 (
   echo $command
   if $quit; then echo QUIT; fi
@@ -836,8 +874,10 @@ tstmsg new/1
 run_compare_test tests/pop3front-maildir-state 'false' 'UIDL' <<END_OF_TEST_RESULTS
 +OK ^M
 1 1^M
+2 this.is.a.very.long.filename.that.should.get.truncated.after.the.X...X^M
 .^M
 new/1
+new/this.is.a.very.long.filename.that.should.get.truncated.after.the.X...XBUGBUGBUGBUG
 END_OF_TEST_RESULTS
 
 run_compare_test tests/pop3front-maildir-state 'false' 'TOP 1 0' <<END_OF_TEST_RESULTS
@@ -847,6 +887,7 @@ Header: foo^M
 ^M
 .^M
 new/1
+new/this.is.a.very.long.filename.that.should.get.truncated.after.the.X...XBUGBUGBUGBUG
 END_OF_TEST_RESULTS
 
 run_compare_test tests/pop3front-maildir-state 'false' 'RETR 1' <<END_OF_TEST_RESULTS
@@ -857,14 +898,17 @@ body^M
 ^M
 .^M
 new/1
+new/this.is.a.very.long.filename.that.should.get.truncated.after.the.X...XBUGBUGBUGBUG
 END_OF_TEST_RESULTS
 
 run_compare_test tests/pop3front-maildir-state 'true' 'UIDL' <<END_OF_TEST_RESULTS
 +OK ^M
 1 1^M
+2 this.is.a.very.long.filename.that.should.get.truncated.after.the.X...X^M
 .^M
 +OK ^M
 cur/1
+cur/this.is.a.very.long.filename.that.should.get.truncated.after.the.X...XBUGBUGBUGBUG
 END_OF_TEST_RESULTS
 
 run_compare_test tests/pop3front-maildir-state 'true' 'TOP 1 0' <<END_OF_TEST_RESULTS
@@ -875,6 +919,7 @@ Header: foo^M
 .^M
 +OK ^M
 cur/1
+cur/this.is.a.very.long.filename.that.should.get.truncated.after.the.X...XBUGBUGBUGBUG
 END_OF_TEST_RESULTS
 
 run_compare_test tests/pop3front-maildir-state 'true' 'RETR 1' <<END_OF_TEST_RESULTS
@@ -886,6 +931,7 @@ body^M
 .^M
 +OK ^M
 cur/1:2,S
+cur/this.is.a.very.long.filename.that.should.get.truncated.after.the.X...XBUGBUGBUGBUG
 END_OF_TEST_RESULTS
 
 
@@ -896,12 +942,12 @@ DATABYTES=123
 
 sfecho <<EOF
 EHLO hostname
-MAIL FROM:<somebody>
-MAIL FROM:<somebody> SIZE
-MAIL FROM:<somebody> SIZE=
-MAIL FROM:<somebody> SIZE=100
-MAIL FROM:<somebody> SIZE=123
-MAIL FROM:<somebody> SIZE=124
+MAIL FROM:<a@example.com>
+MAIL FROM:<a@example.com> SIZE
+MAIL FROM:<a@example.com> SIZE=
+MAIL FROM:<a@example.com> SIZE=100
+MAIL FROM:<a@example.com> SIZE=123
+MAIL FROM:<a@example.com> SIZE=124
 RCPT TO:<nobody@nowhere>
 EOF
 }
@@ -909,14 +955,27 @@ run_compare_test tests/smtpfront-databytes  <<END_OF_TEST_RESULTS
 250-unknown^M
 250-SIZE 123^M
 250-8BITMIME^M
+250-ENHANCEDSTATUSCODES^M
 250 PIPELINING^M
-250 Sender='somebody'.^M
-250 Sender='somebody'.^M
-250 Sender='somebody'.^M
-250 Sender='somebody'.^M
-250 Sender='somebody'.^M
-552 The message would exceed the maximum message size.^M
-503 You must send MAIL FROM: first^M
+250 Sender='a@example.com'.^M
+250 Sender='a@example.com'.^M
+250 Sender='a@example.com'.^M
+250 Sender='a@example.com'.^M
+250 Sender='a@example.com'.^M
+552 5.2.3 The message would exceed the maximum message size.^M
+503 5.5.1 You must send MAIL FROM: first^M
+END_OF_TEST_RESULTS
+
+
+##### Test tests/qmtpfront-echo #####
+
+runtest() {
+printf '6:\ntest\n,13:a@example.com,17:13:b@example.com,,' \
+| $src/qmtpfront-echo 2>/dev/null
+echo
+}
+run_compare_test tests/qmtpfront-echo  <<END_OF_TEST_RESULTS
+20:KReceived 133 bytes.,
 END_OF_TEST_RESULTS
 
 
@@ -934,8 +993,8 @@ END_OF_TEST_RESULTS
 
 runtest() {
 RELAYCLIENT= sfecho <<EOF
-MAIL FROM:<user@remote>
-RCPT TO:<user@local>
+MAIL FROM:<user@example.net>
+RCPT TO:<user@example.com>
 DATA
 Subject: test
 
@@ -943,8 +1002,8 @@ foo
 ..
 bar
 .
-MAIL FROM:<user@remote>
-RCPT TO:<user@local>
+MAIL FROM:<user@example.net>
+RCPT TO:<user@example.com>
 DATA
 Subject: test
 
@@ -956,13 +1015,13 @@ bar
 EOF
 }
 run_compare_test tests/smtpfront-content  <<END_OF_TEST_RESULTS
-250 Sender='user@remote'.^M
-250 Recipient='user@local'.^M
-354 End your message with a period.^M
+250 Sender='user@example.net'.^M
+250 Recipient='user@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 153 bytes.^M
-250 Sender='user@remote'.^M
-250 Recipient='user@local'.^M
-354 End your message with a period.^M
+250 Sender='user@example.net'.^M
+250 Recipient='user@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 154 bytes.^M
 END_OF_TEST_RESULTS
 
@@ -977,8 +1036,8 @@ EOF
 export CVM_SASL_LOGIN=$tmp/cvm
 
 sfqmail <<EOF
-MAIL FROM: <user@remote>
-RCPT TO: <user@remote>
+MAIL FROM: <user@example.com>
+RCPT TO: <user@example.com>
 AUTH LOGIN
 dGVzdHVzZXI=
 dGVzdHBhc3x=
@@ -986,8 +1045,8 @@ AUTH LOGIN
 dGVzdHVzZXI=
 dGVzdHBhc3M=
 AUTH LOGIN
-MAIL FROM: <user@remote>
-RCPT TO: <user@remote>
+MAIL FROM: <user@example.com>
+RCPT TO: <user@example.com>
 EOF
 
 sfqmail << EOF
@@ -999,32 +1058,32 @@ sfqmail <<EOF
 AUTH LOGIN
 dGVzdHVzZXI=
 *
-MAIL FROM: <user@remote>
-RCPT TO: <user@remote>
+MAIL FROM: <user@example.com>
+RCPT TO: <user@example.com>
 EOF
 
 unset CVM_SASL_PLAIN
 }
 run_compare_test tests/smtpfront-auth-login  <<END_OF_TEST_RESULTS
 504 Unrecognized authentication mechanism.^M
-250 Sender accepted.^M
-553 Sorry, that domain isn't in my list of allowed rcpthosts.^M
+250 2.1.0 Sender accepted.^M
+553 5.1.1 Sorry, that domain isn't in my list of allowed rcpthosts.^M
 334 VXNlcm5hbWU6^M
 334 UGFzc3dvcmQ6^M
 501 Authentication failed.^M
 334 VXNlcm5hbWU6^M
 334 UGFzc3dvcmQ6^M
-235 Authentication succeeded.^M
-503 You are already authenticated.^M
-250 Sender accepted.^M
-250 Recipient accepted.^M
+235 2.7.0 Authentication succeeded.^M
+503 5.5.1 You are already authenticated.^M
+250 2.1.0 Sender accepted.^M
+250 2.1.5 Recipient accepted.^M
 334 UGFzc3dvcmQ6^M
-235 Authentication succeeded.^M
+235 2.7.0 Authentication succeeded.^M
 334 VXNlcm5hbWU6^M
 334 UGFzc3dvcmQ6^M
 501 Authentication failed.^M
-250 Sender accepted.^M
-553 Sorry, that domain isn't in my list of allowed rcpthosts.^M
+250 2.1.0 Sender accepted.^M
+553 5.1.1 Sorry, that domain isn't in my list of allowed rcpthosts.^M
 END_OF_TEST_RESULTS
 
 
@@ -1035,31 +1094,31 @@ cat >$tmp/rules <<EOF
 # This selector forces what would normally be a recipient rule to be
 # applied only to the sender.
 :sender
-kone@one:two@two:A
+kone@example.com:two@example.net:A
 # This selector forces what would normally be a sender rule to be
 # applied only to recipients.
 :recipient
-ktwo@two:*:B
+ktwo@example.net:*:B
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<one@one>
-RCPT TO:<three@three>
-RCPT TO:<four@four>
-MAIL FROM:<two@two>
-RCPT TO:<three@three>
-RCPT TO:<four@four>
+MAIL FROM:<one@example.com>
+RCPT TO:<three@example.org>
+RCPT TO:<four@example.biz>
+MAIL FROM:<two@example.net>
+RCPT TO:<three@example.org>
+RCPT TO:<four@example.biz>
 EOF
 
 rm -f $tmp/rules
 }
 run_compare_test tests/rules-selector  <<END_OF_TEST_RESULTS
 250 A^M
-250 Recipient='three@three'.^M
-250 Recipient='four@four'.^M
-250 Sender='two@two'.^M
+250 Recipient='three@example.org'.^M
+250 Recipient='four@example.biz'.^M
+250 Sender='two@example.net'.^M
 250 B^M
 250 B^M
 END_OF_TEST_RESULTS
@@ -1075,13 +1134,13 @@ EOF
 export CVM_SASL_PLAIN=$tmp/cvm
 
 sfqmail <<EOF
-MAIL FROM: <user@remote>
-RCPT TO: <user@remote>
+MAIL FROM: <user@example.com>
+RCPT TO: <user@example.com>
 AUTH PLAIN dGVzdHVzZXIAdGVzdHVzZXIAdGVzdHBhc3x=
 AUTH PLAIN dGVzdHVzZXIAdGVzdHVzZXIAdGVzdHBhc3M=
 AUTH PLAIN dGVzdHVzZXIAdGVzdHVzZXIAdGVzdHBhc3M=
-MAIL FROM: <user@remote>
-RCPT TO: <user@remote>
+MAIL FROM: <user@example.com>
+RCPT TO: <user@example.com>
 EOF
 
 sfqmail << EOF
@@ -1092,8 +1151,8 @@ EOF
 sfqmail <<EOF
 AUTH PLAIN
 *
-MAIL FROM: <user@remote>
-RCPT TO: <user@remote>
+MAIL FROM: <user@example.com>
+RCPT TO: <user@example.com>
 EOF
 
 sfqmail << EOF
@@ -1104,20 +1163,20 @@ unset CVM_SASL_PLAIN
 }
 run_compare_test tests/smtpfront-auth-plain  <<END_OF_TEST_RESULTS
 504 Unrecognized authentication mechanism.^M
-250 Sender accepted.^M
-553 Sorry, that domain isn't in my list of allowed rcpthosts.^M
+250 2.1.0 Sender accepted.^M
+553 5.1.1 Sorry, that domain isn't in my list of allowed rcpthosts.^M
 501 Authentication failed.^M
-235 Authentication succeeded.^M
-503 You are already authenticated.^M
-250 Sender accepted.^M
-250 Recipient accepted.^M
+235 2.7.0 Authentication succeeded.^M
+503 5.5.1 You are already authenticated.^M
+250 2.1.0 Sender accepted.^M
+250 2.1.5 Recipient accepted.^M
 334 ^M
-235 Authentication succeeded.^M
+235 2.7.0 Authentication succeeded.^M
 334 ^M
 501 Authentication failed.^M
-250 Sender accepted.^M
-553 Sorry, that domain isn't in my list of allowed rcpthosts.^M
-235 Authentication succeeded.^M
+250 2.1.0 Sender accepted.^M
+553 5.1.1 Sorry, that domain isn't in my list of allowed rcpthosts.^M
+235 2.7.0 Authentication succeeded.^M
 END_OF_TEST_RESULTS
 
 
@@ -1125,34 +1184,35 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-ksomebody:*::123
+ka@example.com:*::123
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
 EHLO hostname
-MAIL FROM:<somebody>
-MAIL FROM:<somebody> SIZE
-MAIL FROM:<somebody> SIZE=
-MAIL FROM:<somebody> SIZE=100
-MAIL FROM:<somebody> SIZE=123
-MAIL FROM:<somebody> SIZE=124
-RCPT TO:<nobody@nowhere>
+MAIL FROM:<a@example.com>
+MAIL FROM:<a@example.com> SIZE
+MAIL FROM:<a@example.com> SIZE=
+MAIL FROM:<a@example.com> SIZE=100
+MAIL FROM:<a@example.com> SIZE=123
+MAIL FROM:<a@example.com> SIZE=124
+RCPT TO:<nobody@example.net>
 EOF
 }
 run_compare_test tests/rules-databytes  <<END_OF_TEST_RESULTS
 250-unknown^M
 250-SIZE 0^M
 250-8BITMIME^M
+250-ENHANCEDSTATUSCODES^M
 250 PIPELINING^M
 250 OK^M
 250 OK^M
 250 OK^M
 250 OK^M
 250 OK^M
-552 The message would exceed the maximum message size.^M
-503 You must send MAIL FROM: first^M
+552 5.2.3 The message would exceed the maximum message size.^M
+503 5.5.1 You must send MAIL FROM: first^M
 END_OF_TEST_RESULTS
 
 
@@ -1160,19 +1220,19 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-k*:a::1
-k*:b::9999
-k*:c::
+k*:a@example.com::1
+k*:b@example.com::9999
+k*:c@example.com::
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
 EHLO hostname
-MAIL FROM:<somebody>
-RCPT TO:<a>
-RCPT TO:<b>
-RCPT TO:<c>
+MAIL FROM:<somebody@example.net>
+RCPT TO:<a@example.com>
+RCPT TO:<b@example.com>
+RCPT TO:<c@example.com>
 DATA
 testing
 .
@@ -1182,13 +1242,14 @@ run_compare_test tests/rules-databytes2  <<END_OF_TEST_RESULTS
 250-unknown^M
 250-SIZE 0^M
 250-8BITMIME^M
+250-ENHANCEDSTATUSCODES^M
 250 PIPELINING^M
-250 Sender='somebody'.^M
+250 Sender='somebody@example.net'.^M
 250 OK^M
 250 OK^M
 250 OK^M
-354 End your message with a period.^M
-552 Sorry, that message exceeds the maximum message length.^M
+354 End your message with a period on a line by itself.^M
+552 5.2.3 Sorry, that message exceeds the maximum message length.^M
 END_OF_TEST_RESULTS
 
 
@@ -1196,23 +1257,23 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-dd:*
-zz:*
-kk:*
-d*:d
-z*:z
-k*:k
+dd@example.com:*
+zz@example.com:*
+kk@example.com:*
+d*:d@example.com
+z*:z@example.com
+k*:k@example.com
 EOF
 
 MAILRULES=$tmp/rules
 
 sfecho <<EOF
-MAIL FROM:<d>
-MAIL FROM:<z>
-MAIL FROM:<k>
-RCPT TO:<d>
-RCPT TO:<z>
-RCPT TO:<k>
+MAIL FROM:<d@example.com>
+MAIL FROM:<z@example.com>
+MAIL FROM:<k@example.com>
+RCPT TO:<d@example.com>
+RCPT TO:<z@example.com>
+RCPT TO:<k@example.com>
 EOF
 
 rm -f $tmp/rules
@@ -1227,6 +1288,86 @@ run_compare_test tests/rules-defaultmsg  <<END_OF_TEST_RESULTS
 END_OF_TEST_RESULTS
 
 
+##### Test tests/smtpfront-require-auth #####
+
+runtest() {
+export CVM_SASL_PLAIN=$tmp/cvm
+export REQUIRE_AUTH=1
+
+sfqmail <<EOF
+MAIL FROM: <user@example.net>
+RCPT TO: <user@example.net>
+AUTH PLAIN dGVzdHVzZXIAdGVzdHVzZXIAdGVzdHBhc3M=
+MAIL FROM: <user@example.net>
+RCPT TO: <user@example.net>
+EOF
+
+unset CVM_SASL_PLAIN
+unset REQUIRE_AUTH
+}
+run_compare_test tests/smtpfront-require-auth  <<END_OF_TEST_RESULTS
+530 5.7.1 You must authenticate first.^M
+503 5.5.1 You must send MAIL FROM: first^M
+235 2.7.0 Authentication succeeded.^M
+250 2.1.0 Sender accepted.^M
+250 2.1.5 Recipient accepted.^M
+END_OF_TEST_RESULTS
+
+
+##### Test tests/smtpfront-reject #####
+
+runtest() {
+local msg="$1"
+env SMTPREJECT="$msg" $src/smtpfront-reject 2>/dev/null <<EOF
+HELO nobody
+EHLO somebody
+MAIL FROM:<somewhere>
+RCPT TO:<elsewhere>
+EOF
+}
+run_compare_test tests/smtpfront-reject 'rej' <<END_OF_TEST_RESULTS
+220 unknown mailfront ESMTP^M
+250 unknown^M
+250-unknown^M
+250-SIZE 0^M
+250-8BITMIME^M
+250-ENHANCEDSTATUSCODES^M
+250 PIPELINING^M
+451 rej^M
+503 5.5.1 You must send MAIL FROM: first^M
+END_OF_TEST_RESULTS
+
+run_compare_test tests/smtpfront-reject '-rej' <<END_OF_TEST_RESULTS
+220 unknown mailfront ESMTP^M
+250 unknown^M
+250-unknown^M
+250-SIZE 0^M
+250-8BITMIME^M
+250-ENHANCEDSTATUSCODES^M
+250 PIPELINING^M
+553 rej^M
+503 5.5.1 You must send MAIL FROM: first^M
+END_OF_TEST_RESULTS
+
+
+##### Test tests/smtpfront-quotes #####
+
+runtest() {
+sfecho <<EOF
+MAIL FROM:<"me, myself, and I"@example.net>
+RCPT TO:<"you, yourself, and you"@example.com>
+RCPT TO:<him\,himself@example.com>
+RCPT TO:<@somewhere,@elsewhere:two@example.com>
+EOF
+}
+run_compare_test tests/smtpfront-quotes  <<END_OF_TEST_RESULTS
+250 Sender='me, myself, and I@example.net'.^M
+250 Recipient='you, yourself, and you@example.com'.^M
+250 Recipient='him,himself@example.com'.^M
+250 Recipient='two@example.com'.^M
+END_OF_TEST_RESULTS
+
+
 ##### Test tests/pop3front-auth-userpass #####
 
 runtest() {
@@ -1260,7 +1401,7 @@ export PATTERNS
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<nobody>
+RCPT TO:<nobody@example.com>
 DATA
 before
 
@@ -1276,7 +1417,7 @@ EOF
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<nobody>
+RCPT TO:<nobody@example.com>
 DATA
 before
 
@@ -1288,13 +1429,13 @@ rm -f $tmp/patterns
 }
 run_compare_test tests/patterns-after  <<END_OF_TEST_RESULTS
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 554 This message contains prohibited content^M
 
 250 Sender=''.^M
-250 Recipient='nobody'.^M
-354 End your message with a period.^M
+250 Recipient='nobody@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 142 bytes.^M
 END_OF_TEST_RESULTS
 
@@ -1303,23 +1444,23 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-ka:b:K1:::MAXHOPS=1
+ka@example.com:b@example.com:K1:::MAXHOPS=1
 EOF
 
 MAILRULES=$tmp/rules
 export MAILRULES
 
 sfecho <<EOF
-MAIL FROM:<a>
-RCPT TO:<b>
+MAIL FROM:<a@example.com>
+RCPT TO:<b@example.com>
 DATA
 Received: hop1
 Received: hop2
 .
 EOF
 MAILRULES=$tmp/rules sfecho <<EOF
-MAIL FROM:<a>
-RCPT TO:<c>
+MAIL FROM:<a@example.com>
+RCPT TO:<c@example.com>
 DATA
 Received: hop1
 Received: hop1
@@ -1329,13 +1470,13 @@ EOF
 rm -f $tmp/rules
 }
 run_compare_test tests/rules-maxhops  <<END_OF_TEST_RESULTS
-250 Sender='a'.^M
+250 Sender='a@example.com'.^M
 250 K1^M
-354 End your message with a period.^M
-554 This message is looping, too many hops.^M
-250 Sender='a'.^M
-250 Recipient='c'.^M
-354 End your message with a period.^M
+354 End your message with a period on a line by itself.^M
+554 5.6.0 This message is looping, too many hops.^M
+250 Sender='a@example.com'.^M
+250 Recipient='c@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 158 bytes.^M
 END_OF_TEST_RESULTS
 
@@ -1347,8 +1488,8 @@ MAXHOPS=1
 export MAXHOPS
 
 sfecho <<EOF
-MAIL FROM:<somebody>
-RCPT TO:<nobody@nowhere>
+MAIL FROM:<somebody@example.com>
+RCPT TO:<nobody@example.org>
 DATA
 Delivered-To: foo
 .
@@ -1357,8 +1498,8 @@ EOF
 echo
 
 sfecho <<EOF
-MAIL FROM:<somebody>
-RCPT TO:<nobody@nowhere>
+MAIL FROM:<somebody@example.com>
+RCPT TO:<nobody@example.org>
 DATA
 Delivered-To: foo
 Delivered-To: foo
@@ -1366,15 +1507,15 @@ Delivered-To: foo
 EOF
 }
 run_compare_test tests/smtpfront-looping-delivered-to  <<END_OF_TEST_RESULTS
-250 Sender='somebody'.^M
-250 Recipient='nobody@nowhere'.^M
-354 End your message with a period.^M
+250 Sender='somebody@example.com'.^M
+250 Recipient='nobody@example.org'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 146 bytes.^M
 
-250 Sender='somebody'.^M
-250 Recipient='nobody@nowhere'.^M
-354 End your message with a period.^M
-554 This message is looping, too many hops.^M
+250 Sender='somebody@example.com'.^M
+250 Recipient='nobody@example.org'.^M
+354 End your message with a period on a line by itself.^M
+554 5.6.0 This message is looping, too many hops.^M
 END_OF_TEST_RESULTS
 
 
@@ -1389,29 +1530,29 @@ d*:*:DD
 EOF
 
 cat >$tmp/list <<EOF
-a@a
-@c
+a@example.net
+@example.com
 EOF
 
 cat >$tmp/atlist <<EOF
-b
+example.org
 EOF
 
 MAILRULES=$tmp/rules $src/smtpfront-echo <<EOF 2>/dev/null | tail -n +2
-MAIL FROM:<nobody@nowhere>
-RCPT TO:<a@a>
-RCPT TO:<b@a>
-RCPT TO:<a@b>
-RCPT TO:<b@B>
-RCPT TO:<c@c>
-RCPT TO:<c@C>
-RCPT TO:<d@d>
+MAIL FROM:<nobody@example.com>
+RCPT TO:<a@example.net>
+RCPT TO:<b@example.net>
+RCPT TO:<a@example.org>
+RCPT TO:<b@example.org>
+RCPT TO:<c@example.com>
+RCPT TO:<c@Example.COM>
+RCPT TO:<d@example.biz>
 EOF
 
 rm -f $tmp/rules $tmp/list $tmp/atlist
 }
 run_compare_test tests/rules-rcptlist  <<END_OF_TEST_RESULTS
-250 Sender='nobody@nowhere'.^M
+250 Sender='nobody@example.com'.^M
 250 LIST^M
 553 DD^M
 250 ATLIST^M
@@ -1509,42 +1650,46 @@ END_OF_TEST_RESULTS
 ##### Test tests/smtpfront-bad-bounce #####
 
 runtest() {
+# Note: this test no longer tests anything significant.
+
 sfecho <<EOF
-MAIL FROM:<not@bounce>
-RCPT TO:<addr1@domain>
-RCPT TO:<addr2@domain>
+MAIL FROM:<notbounce@example.com>
+RCPT TO:<addr1@example.com>
+RCPT TO:<addr2@example.com>
 DATA
 .
 EOF
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<addr1@domain>
+RCPT TO:<addr1@example.com>
 DATA
 .
 EOF
 
 sfecho <<EOF
 MAIL FROM:<>
-RCPT TO:<addr1@domain>
-RCPT TO:<addr2@domain>
+RCPT TO:<addr1@example.com>
+RCPT TO:<addr2@example.com>
 DATA
+.
 EOF
 }
 run_compare_test tests/smtpfront-bad-bounce  <<END_OF_TEST_RESULTS
-250 Sender='not@bounce'.^M
-250 Recipient='addr1@domain'.^M
-250 Recipient='addr2@domain'.^M
-354 End your message with a period.^M
+250 Sender='notbounce@example.com'.^M
+250 Recipient='addr1@example.com'.^M
+250 Recipient='addr2@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 128 bytes.^M
 250 Sender=''.^M
-250 Recipient='addr1@domain'.^M
-354 End your message with a period.^M
+250 Recipient='addr1@example.com'.^M
+354 End your message with a period on a line by itself.^M
 250 Received 128 bytes.^M
 250 Sender=''.^M
-250 Recipient='addr1@domain'.^M
-550 Bounce messages should have a single recipient.^M
-550 Bounce messages should have a single recipient.^M
+250 Recipient='addr1@example.com'.^M
+250 Recipient='addr2@example.com'.^M
+354 End your message with a period on a line by itself.^M
+250 Received 128 bytes.^M
 END_OF_TEST_RESULTS
 
 
@@ -1555,7 +1700,7 @@ sfecho2() {
 $src/smtpfront-echo 2>&1 >/dev/null <<EOF | \
 	sed -n -e 's/^.* Received: //p'
 MAIL FROM:<>
-RCPT TO:<test>
+RCPT TO:<test@example.com>
 DATA
 .
 EOF
@@ -1583,7 +1728,7 @@ $src/smtpfront-echo 2>&1 >/dev/null <<EOF | \
 	sed -n -e 's/^.* Received: //p'
 EHLO hh
 MAIL FROM:<>
-RCPT TO:<test>
+RCPT TO:<test@example.com>
 DATA
 .
 EOF
@@ -1646,18 +1791,18 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-ka@a:*:ONE\nTWO
-ka@b:*:ONE\:TWO
-ka@c:*:ONE\\\\TWO:
+ka@example.com:*:ONE\nTWO
+ka@example.net:*:ONE\:TWO
+ka@example.org:*:ONE\\\\TWO:
 EOF
 
 MAILRULES=$tmp/rules
 export MAILRULES
 
 sfecho <<EOF
-MAIL FROM:<a@a>
-MAIL FROM:<a@b>
-MAIL FROM:<a@c>
+MAIL FROM:<a@example.com>
+MAIL FROM:<a@example.net>
+MAIL FROM:<a@example.org>
 EOF
 
 rm -f $tmp/rules
@@ -1674,8 +1819,8 @@ END_OF_TEST_RESULTS
 
 runtest() {
 cat >$tmp/rules <<EOF
-ka@a:*:K1
-k*@a:*:K2
+ka@example.com:*:K1
+k*@example.com:*:K2
 kc@*:*:K3
 k*:*:K4
 EOF
@@ -1684,11 +1829,12 @@ MAILRULES=$tmp/rules
 export MAILRULES
 
 sfecho <<EOF
-MAIL FROM:<a@a>
-MAIL FROM:<b@a>
-MAIL FROM:<c@b>
-MAIL FROM:<d@d>
+MAIL FROM:<a@example.com>
+MAIL FROM:<b@example.com>
+MAIL FROM:<c@example.net>
+MAIL FROM:<d@example.org>
 MAIL FROM:<>
+MAIL FROM:<1@2@example.com@example.com>
 EOF
 
 rm -f $tmp/rules
@@ -1699,6 +1845,7 @@ run_compare_test tests/rules-asterisk  <<END_OF_TEST_RESULTS
 250 K3^M
 250 K4^M
 250 K4^M
+250 K2^M
 END_OF_TEST_RESULTS
 
 
